@@ -1,7 +1,13 @@
 from fontTools.colorLib.builder import buildCOLR
 from fontTools.ttLib import TTFont, newTable
 from fontTools.ttLib.tables import otTables as ot
-from fontTools.varLib import build, build_many, load_designspace, _add_COLR
+from fontTools.varLib import (
+    build,
+    build_many,
+    load_designspace,
+    _add_COLR,
+    addGSUBFeatureVariations,
+)
 from fontTools.varLib.errors import VarLibValidationError
 import fontTools.varLib.errors as varLibErrors
 from fontTools.varLib.models import VariationModel
@@ -169,6 +175,15 @@ class BuildTest(unittest.TestCase):
             expected_ttx_name="Build",
         )
 
+    def test_varlib_build_ttf_reuse_nameid_2(self):
+        """Instances at the default location can reuse name ID 2 or 17."""
+        self._run_varlib_build_test(
+            designspace_name="BuildReuseNameId2",
+            font_name="TestFamily",
+            tables=["fvar"],
+            expected_ttx_name="BuildReuseNameId2",
+        )
+
     def test_varlib_build_no_axes_ttf(self):
         """Designspace file does not contain an <axes> element."""
         ds_path = self.get_test_input("InterpolateLayout3.designspace")
@@ -331,12 +346,38 @@ class BuildTest(unittest.TestCase):
             post_process_master=add_rclt,
         )
 
+    def test_varlib_build_feature_variations_without_latn_dflt_feature(self):
+        """Test that when a script does not have a dflt language, it gets one
+        when we later add variations, we can attach them to it."""
+
+        def add_features(font, savepath):
+            features = """
+            languagesystem DFLT dflt;
+            languagesystem latn dflt;
+            languagesystem latn CAT;
+
+            feature locl {
+                script latn;
+                # Intentionally skip defining anything for `language dflt;`.
+                language CAT;
+                sub uni0061 by uni0061;
+            } locl;
+            """
+            addOpenTypeFeaturesFromString(font, features)
+            font.save(savepath)
+
+        self._run_varlib_build_test(
+            designspace_name="FeatureVars",
+            font_name="TestFamily",
+            tables=["GSUB"],
+            expected_ttx_name="FeatureVars_latn_dflt_var",
+            save_before_dump=True,
+            post_process_master=add_features,
+        )
+
     def test_varlib_gvar_explicit_delta(self):
         """The variable font contains a composite glyph odieresis which does not
-        need a gvar entry, because all its deltas are 0, but it must be added
-        anyway to work around an issue with macOS 10.14.
-
-        https://github.com/fonttools/fonttools/issues/1381
+        need a gvar entry.
         """
         test_name = "BuildGvarCompositeExplicitDelta"
         self._run_varlib_build_test(
@@ -999,6 +1040,42 @@ Expected to see .ScriptCount==1, instead saw 0""",
             save_before_dump=True,
         )
 
+    def test_varlib_build_variable_cff2_with_empty_sparse_glyph(self):
+        # https://github.com/fonttools/fonttools/issues/3233
+        self._run_varlib_build_test(
+            designspace_name="SparseCFF2",
+            font_name="SparseCFF2",
+            tables=["GlyphOrder", "CFF2", "fvar", "hmtx", "HVAR"],
+            expected_ttx_name="SparseCFF2-VF",
+            save_before_dump=True,
+        )
+
+    def test_varlib_addGSUBFeatureVariations(self):
+        ttx_dir = self.get_test_input("master_ttx_interpolatable_ttf")
+
+        ds = DesignSpaceDocument.fromfile(
+            self.get_test_input("FeatureVars.designspace")
+        )
+        for source in ds.sources:
+            ttx_dump = TTFont()
+            ttx_dump.importXML(
+                os.path.join(
+                    ttx_dir, os.path.basename(source.filename).replace(".ufo", ".ttx")
+                )
+            )
+            source.font = ttx_dump
+
+        varfont, _, _ = build(ds, exclude=["GSUB"])
+        assert "GSUB" not in varfont
+
+        addGSUBFeatureVariations(varfont, ds)
+        assert "GSUB" in varfont
+
+        tables = ["fvar", "GSUB"]
+        expected_ttx_path = self.get_test_output("FeatureVars.ttx")
+        self.expect_ttx(varfont, expected_ttx_path, tables)
+        self.check_ttx_dump(varfont, expected_ttx_path, tables, ".ttf")
+
 
 def test_load_masters_layerName_without_required_font():
     ds = DesignSpaceDocument()
